/**************************************************************************
 * scannerdevice.h - class declaration of ScannerDevice class.
 * Copyright (C) 2003-2005 Lexmark International, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is  distributed in the hope that it will  be useful,
 * but WITHOUT ANY WARRANTY;  without  even the implied  warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License at http://www.gnu.org/licenses/licenses.html
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation,Inc.,59 Temple Place - Suite 330,Boston,MA 02111-1307,USA.
 *
 **************************************************************************/
#ifndef SCANNER_DEVICE_H
#define SCANNER_DEVICE_H

#include "portmonitor.h"
#include "clock.h"

class ScanErrorManager;
class ScanErrorInterface;
class ScanCommunication;
class ScanColorConvert;

class ScannerDevice
{
	public:

	enum SD_ErrCode {
		SD_OK,
		SD_COMMUNICATION_ERROR,
		SD_MODULES_UNINITIALIZED,
		SD_INVALID_STATE,
		SD_DEVICE_WARMING_UP,
		SD_SCANNER_NOT_LOCKED,
		SD_DEVICE_OFF,
		SD_COPIER_NOT_LOCKED,
		SD_SCAN_STOPPED,
		SD_MEMORY_OUT,
		SD_UNEXPECTED_ERROR,
		SD_ADF_PAPER_JAM
	};
	
	enum SD_Channels { SD_MONO_CHANNEL, SD_RGB_CHANNEL };
	enum SD_BitsPerChannel { SD_TWO_BPC, SD_EIGHT_BPC };
	enum SD_HalftoneMode { SD_THRESHOLD, SD_ERROR_DIFFUSION };
	enum SD_CompressionMode { SD_UNCOMPRESSED, SD_LOSSLESS_JPG };
	enum SD_Resolution { SD_75_DPI, SD_100_DPI, SD_150_DPI, SD_200_DPI, SD_300_DPI,
       			     SD_600_DPI, SD_1200_DPI, SD_2400_DPI, SD_4800_DPI, SD_9600_DPI, SD_19200_DPI };
	enum SD_FilterLevel { SD_NO_FILTER, SD_HIGH_BLUR, SD_MODERATE_BLUR, SD_LOW_BLUR,
			      SD_LOW_SHARPEN, SD_MODERATE_SHARPEN, SD_HIGH_SHARPEN,
			      SD_SEVERE_SHARPEN };
	enum SD_Clipping { SD_NO_CLIPPING, SD_1_PERCENT, SD_2_PERCENT, SD_3_PERCENT,
                           SD_5_PERCENT, SD_8_PERCENT, SD_10_PERCENT, SD_12_PERCENT,		
                           SD_15_PERCENT, SD_20_PERCENT, SD_25_PERCENT, SD_35_PERCENT,		
                           SD_50_PERCENT, SD_65_PERCENT, SD_80_PERCENT, SD_100_PERCENT };		
	enum SD_ContentType { SD_NO_CONTENT_TYPE, SD_TEXT_DOCUMENT, SD_PHOTO, SD_MIXED };
	enum SD_ImageQuality { SD_STANDARD, SD_FOR_SPEED, SD_FOR_QUALITY };
	
	typedef struct {
		SD_Channels channels;
		SD_BitsPerChannel bitsperchannel;
		SD_HalftoneMode halftonemode;
		SD_CompressionMode compressionmode;
		SD_Resolution horizontaldpi;
		SD_Resolution verticaldpi;
		unsigned int horizontaloffset;
		unsigned int verticaloffset;
		unsigned int width;
		unsigned int height;
		SD_ContentType contenttype;
		SD_ImageQuality imagequality;
	} SD_ScanProp;

	public:
	
	ScannerDevice(
		PortMonitor * ScannerPort,
		ScanErrorInterface * ScanError,
		char * ScannerModel);
	~ScannerDevice();

	SD_ErrCode SD_StartScan(
		SD_ScanProp ScanProp);

	long SD_ReadScanData(
		unsigned char * data,
		unsigned long size,
		int * eof);
	SD_ErrCode SD_EndScan();
	SD_ErrCode SD_CancelScan();
  	SD_ErrCode SD_SetScanApps(unsigned char * buffer);
  	long SD_XScaleData(int numData);
	unsigned long ScannerDevice::SD_YScaleData(
		unsigned char *data, 
		unsigned long size);
	long SD_MoreScaleData(unsigned char *data, long size);
	SD_ErrCode SD_GetADFStatus(unsigned char * adfStatus);
	// Copy methods --- to be determined
	// SD_ErrCode SD_StartCopy();
	// SD_ErrCode SD_ReadCopyData();
	// SD_ErrCode SD_EndCopy();
	// SD_ErrCode SD_CancelCopy();

	
 private:

	char m_ScannerModel[255];
	ScanCommunication * m_SComm;
	ScanErrorManager * m_Error;
	ScanColorConvert * m_SColor;
	unsigned char m_ActiveState;
  SD_ErrCode SD_MapScanCommError();
  void SD_NotifyError(SD_ErrCode Error);
};

#endif // SCANNER_DEVICE_H
